<?php

/**
 * restore.sh が作ったlockfile がある場合は exit 1
 */
require_once('/opt/passlogic/apps/lib/passlogic_lock.php');
if (PasslogicLock::cron_exists())
{
	exit(1);
}

/**
 * settings
 */
define('UserLdapCmd_CHECK_DB_STATUS', 0);
require_once('/opt/passlogic/apps/lib/settings/global_setting.php');
require_once(LIB_DIR . '/settings/server_setting.php');
require_once(LIB_DIR . "/language.inc.php");
require_once(LIB_DIR . "/ldap.inc.php");
$Language = new Language;

class ldapsync {

	private $domain = 'local';
	private $plLog = NULL;
	private $ascii_default_mapping = array(
	    'uid' => 'ldapAttrUid',
	    'policy' => 'ldapAttrPolicy',
	    'group1' => 'ldapAttrGroup1',
	    'group2' => 'ldapAttrGroup2',
	    'group3' => 'ldapAttrGroup3',
	    'group4' => 'ldapAttrGroup4',
	    'group5' => 'ldapAttrGroup5',
	    'uemail' => 'ldapAttrUemail',
	    'uexpiry' => 'ldapAttrUexpiry',
	);
	private $multibyte_default_mapping = array(
	    'uname' => 'ldapAttrUname',
	    'employee_number' => 'ldapAttrEmployee_number',
	    'section' => 'ldapAttrSection',
	    'phone' => 'ldapAttrPhone',
	    'ucomment' => 'ldapAttrUcomment',
	    'attribute1' => 'ldapAttrAttribute1',
	    'attribute2' => 'ldapAttrAttribute2',
	    'attribute3' => 'ldapAttrAttribute3',
	    'attribute4' => 'ldapAttrAttribute4',
	    'attribute5' => 'ldapAttrAttribute5',
	    'attribute6' => 'ldapAttrAttribute6',
	    'attribute7' => 'ldapAttrAttribute7',
	    'attribute8' => 'ldapAttrAttribute8',
	    'attribute9' => 'ldapAttrAttribute9',
	    'attribute10' => 'ldapAttrAttribute10',
	    'sslvpnsso1' => 'ldapAttrSslvpnsso1',
	    'sslvpnsso2' => 'ldapAttrSslvpnsso2',
	    'sslvpnsso3' => 'ldapAttrSslvpnsso3',
	    'sslvpnsso4' => 'ldapAttrSslvpnsso4',
	    'sslvpnsso5' => 'ldapAttrSslvpnsso5',
	    'sslvpnsso6' => 'ldapAttrSslvpnsso6',
	    'sslvpnsso7' => 'ldapAttrSslvpnsso7',
	    'sslvpnsso8' => 'ldapAttrSslvpnsso8',
	    'sslvpnsso9' => 'ldapAttrSslvpnsso9',
	    'sslvpnsso10' => 'ldapAttrSslvpnsso10',
	    'param1' => 'ldapAttrParam1',
	    'param2' => 'ldapAttrParam2',
	    'param3' => 'ldapAttrParam3',
	    'param4' => 'ldapAttrParam4',
	    'param5' => 'ldapAttrParam5',
	    'param6' => 'ldapAttrParam6',
	    'param7' => 'ldapAttrParam7',
	    'param8' => 'ldapAttrParam8',
	    'param9' => 'ldapAttrParam9',
	    'param10' => 'ldapAttrParam10',
	);

	/**
	 * 
	 * コンストラクタ
	 * 
	 */
	function __construct() {
		global $Language;
		$this->message = $Language->locale == 'ja' ? 'message_ja' : 'message';

		$this->plLog = new PasslogicLog();
		$this->api_util = new passlogic_api_util();
	}

	/**
	 * LDAPサーバとの接続処理
	 * 
	 * @param string $server_name 接続先のLDAPサーバのホスト名(もしくはIPアドレス)
	 * @param array $ldap_config LDAP接続設定の連想配列
	 * @return 成功の場合は LDAP BINDオブジェクト、失敗の場合は NULL
	 */
	private function bind_LDAP_server($server_name, $ldap_config) {
		$protocol = ( $ldap_config['ldapSecure'] ? 'ldaps' : 'ldap' );
		$port = $ldap_config['ldapPort'];

		// 接続用URIの構文チェック(実際の接続はこの時点では未実施)
		$ldap_connection = ldap_connect("{$protocol}://{$server_name}:{$port}");
		if ($errnum = ldap_errno($ldap_connection)) {
			$errstr = ldap_err2str($errnum);
			$this->plLog->log("31213", '', $this->domain, $this->tenant, array('additional_message' => "{$server_name} : {$errstr}"));
			return NULL;
		}

		// LDAPサーバ接続用の各種パラメータ設定
		ldap_set_option($ldap_connection, LDAP_OPT_NETWORK_TIMEOUT, LDAP_NETWORK_TIMEOUT);

		if (!ldap_set_option($ldap_connection, LDAP_OPT_PROTOCOL_VERSION, 3)) {
			$errnum = ldap_errno($ldap_connection);
			$errstr = ldap_err2str($errnum);
			$this->plLog->log("31214", '', $this->domain, $this->tenant, array('additional_message' => "{$server_name} : {$errstr}"));
			return NULL;
		}

		if ($ldap_config['ldapType'] == 'ActiveDirectory') {
			ldap_get_option($ldap_connection, LDAP_OPT_REFERRALS, $ret_params);
			if ($ret_params) {
				if (!ldap_set_option($ldap_connection, LDAP_OPT_REFERRALS, 0)) {
					$errnum = ldap_errno($ldap_connection);
					$errstr = ldap_err2str($errnum);
					$this->plLog->log("31215", '', $this->domain, $this->tenant, array('additional_message' => "{$server_name} : {$errstr}"));
					return NULL;
				}
			}
		}

		// LDAPサーバ接続
		@ldap_bind($ldap_connection, $ldap_config['ldapBinddn'], $ldap_config['ldapBindpasswd']);
		if ($errnum = ldap_errno($ldap_connection)) {
			$errstr = ldap_err2str($errnum);
			$this->plLog->log("31216", '', $this->domain, $this->tenant, array('additional_message' => "{$server_name} : {$errstr}"));
			return NULL;
		}

		return $ldap_connection;
	}

	private function dbstatus() {
		if (file_exists(DATA_DIR . '/conf/flag/other_server')) {
			switch (file_get_contents(DATA_DIR . '/conf/flag/other_server')) {
				case server_0:
					$db = '0';
					break;
				case server_1:
					$db = '1';
					break;
				default: // ここには入らない
					return false;
			}
		} else {
			$db = '0';
		}
		
		$response_array = $this->api_util->dbstatus($db);
		switch ($response_array['code']) {
			case 10000:
			case 10003:
			case 10004:
			case 10005:
				return false;
			case 10001:
			case 10002:
				return true;
			default:
				return false;
		}
	}


	/**
	 * 外部から呼び出すメイン実行処理部分
	 * 
	 * @param string $targetLdapDomain 同期対象のドメイン名
	 * @param string $targetTenant  同期対象のドメインが所属するテナント名
	 * @return
	 */
	public function main($targetLdapDomain, $targetTenant) {
		$this->tenant = $targetTenant;
		// configデータベース
		$passlogic_config = new passlogic_config(getPDO());
		// LDAP ID同期ドメインの一覧から該当ドメインのものを読み込み
		$ldap_config = $passlogic_config->get_config('domain', $targetLdapDomain, $targetTenant);

		if ($ldap_config['domainname']) {
			$this->domain = $ldap_config['domainname'];
		}

		if (UserLdapCmd_CHECK_DB_STATUS && !$this->dbstatus()) {
			$this->plLog->log("31219", '', $this->domain, $this->tenant);
			exit;
		}
		
		// 処理開始(31201: Start exporting user data to PassLogic from LDAP.)
		$this->plLog->log("31201", '', $this->domain, $this->tenant);

		if (!$targetLdapDomain) {
			$this->plLog->log("31211", '', $this->domain, $this->tenant);
			exit;
		}

		if ($ldap_config and is_array($ldap_config)) {
			$crypt = new Crypt();
			$ldap_config['ldapBindpasswd'] = $crypt->decrypt_string($ldap_config['ldapBindpasswd'], $ldap_config['domainname']);
		} else {
			$this->plLog->log("31212", '', $this->domain, $this->tenant);
			exit;
		}

		// 対象のLDAPサーバに接続
		$ldap_servers = array(
		    $ldap_config['ldapHostname'],
		    $ldap_config['ldapHostname2'],
		);
		$ldap_connection = NULL;
		foreach ($ldap_servers as $ldap_server) {
			if (!$ldap_server) {
				continue;
			}
			if ($ldap_connection = $this->bind_LDAP_server($ldap_server, $ldap_config)) {
				break;
			}
		}
		if (!$ldap_connection) {
			$this->plLog->log("31216", '', $this->domain, $this->tenant, array('additional_message' => 'no LDAP server can be binded.'));
			exit;
		}

		// LDAPサーバで検索する際、取得対象の属性名の一覧を作成
		// - ${ascii,multibyte}_default_mappingのうち、属性名の指定があるものを
		//   設定ファイルから取得してきて、データベース列名とLDAP属性名の新しいマッピング連想配列を作成
		$search_attributes = array();
		$ascii_mapping = array();
		foreach ($this->ascii_default_mapping as $column_name => $attribute_name) {
			if ($ldap_config[$attribute_name]) {
				$search_attributes[] = $ldap_config[$attribute_name];
				$ascii_mapping[$column_name] = strtolower($ldap_config[$attribute_name]);
			}
		}
		$multibyte_mapping = array();
		foreach ($this->multibyte_default_mapping as $column_name => $attribute_name) {
			if ($ldap_config[$attribute_name]) {
				$search_attributes[] = $ldap_config[$attribute_name];
				$multibyte_mapping[$column_name] = strtolower($ldap_config[$attribute_name]);
			}
		}
		// 別々のデータベース列名から、同一のLDAP属性名を参照する場合があるので注意
		$search_attributes = array_values(array_unique($search_attributes));

		// LDAP検索処理
		$ldapsearch = ldap_search($ldap_connection, $ldap_config['ldapTopObject'], $ldap_config['ldapFilter'], $search_attributes);
		if ($errnum = ldap_errno($ldap_connection)) {
			$errstr = ldap_err2str($errnum);
			$this->plLog->log("31217", '', $this->domain, $this->tenant, array('additional_message' => $errstr));
			exit;
		}

		$ldap_entries = ldap_get_entries($ldap_connection, $ldapsearch);
		if ($errnum = ldap_errno($ldap_connection)) {
			$errstr = ldap_err2str($errnum);
			$this->plLog->log("31218", '', $this->domain, $this->tenant, array('additional_message' => $errstr));
			exit;
		}

		// 取得したLDAPユーザデータを格納するCSVファイルとCSVコンフィグを作成（初期化）
		$uniqid = uniqid();
		$config_file = TMP_DIR . "/userLdapCmd_configfile_" . $uniqid;
		file_put_contents($config_file, '');
		$tmpfile_name = TMP_DIR . "/userLdapCmd_csv_" . $uniqid;
		file_put_contents($tmpfile_name, '');

		// 取り込みデータのカラム作成
		$column_names = array_merge(array_keys($ascii_mapping), array_keys($multibyte_mapping));
		// ポリシーは必須になるので、なければ追加
		if (!in_array('policy', $column_names)) {
			$column_names[] = 'policy';
		}

		// 共通初期パターン
		if($ldap_config['isSetSharedPattern'] && $ldap_config['sharedPattern']) {
			$column_names[] = 'secret_pattern';
			$ldap_config['sharedPattern'] = $crypt->decrypt_string($ldap_config['sharedPattern'], $ldap_config['domainname']);
		}

		// CSVインポート用の設定ファイルを作成
		$csv_config = "0\n";
		$csv_config .= ( $ldap_config['ldapAddactMail'] ? '1' : '0' ) . "\n";
		$csv_config .= "delflag,uid,domain,";
		foreach ($column_names as $column_name) {
			switch ($column_name) {
				case 'uid':
					break;
				default:
					$csv_config .= "{$column_name},";
			}
		}
		file_put_contents($config_file, preg_replace('/\,$/', '', $csv_config));

		// 完全同期の場合の比較用uidリストを初期化
		if ($ldap_config['ldapRunMode'] == 'sync') {
			$ldap_users_uid = array();
		}

		// LDAP検索結果をもとに、CSVに格納用のユーザ情報の配列を作成
		$num_entries = $ldap_entries['count'];
		for ($i = 0; $i < $num_entries; $i++) {
			// 1ユーザずつコード変換
			$ldap_user = array();
			foreach ($ldap_entries[$i] as $n => $v) {
				$name = strtolower($n);
				$value = NULL;

				switch (strtolower($name)) {
					case 'dn':
						$value = $v;
						break;
					case 'objectguid':
						$value = base64_encode($v[0]);
						break;
					default:
						$value = $v[0];
				}

				foreach ($ascii_mapping as $column_name => $attribute_name) {
					if ($attribute_name == $name) {
						$ldap_user[$column_name] = $value;
					}
				}
				foreach ($multibyte_mapping as $column_name => $attribute_name) {
					if ($attribute_name == $name) {
						$ldap_user[$column_name] = mb_convert_encoding($value, "sjis-win", "UTF-8");
					}
				}
			}

			// $ldap_userに格納完了したので、CSV形式に変換
			// 取り込み（編集）ユーザデータをCSVに書き込む
			// 先頭の要素は、削除フラグ、uid、domainで固定
			$csv_data = '"",';
			$csv_data .= '"' . preg_replace("/\"/", "\"\"", $ldap_user['uid']) . '",';
			$csv_data .= '"' . $this->domain . '",';
			// その後は列順に従って要素を配置
			foreach ($column_names as $column_name) {
				switch ($column_name) {
					case 'uid':
						break;
					case 'policy':
						$policy0 = ( $ldap_user[$column_name] ? $ldap_user[$column_name] : $ldap_config['ldapDefaultPolicy'] );
						$csv_data .= '"' . $policy0 . '",';
						break;
					case 'expire':
						$csv_data .= '"' . $ldap_user[$column_name] . '",';
						break;
					case 'secret_pattern':
						$csv_data .= '"' . $ldap_config['sharedPattern'] . '",';
					default:
						$csv_data .= '"' . preg_replace("/\"/", "\"\"", $ldap_user[$column_name]) . '",';
				}
			}

			// 1ユーザ書き込み
			file_put_contents($tmpfile_name, preg_replace('/\,$/', '', $csv_data) . "\n", FILE_APPEND);

			// 完全同期の場合の比較用uidリスト
			if ($ldap_config['ldapRunMode'] == 'sync') {
				$ldap_users_uid[] = $ldap_user['uid'];
			}
		}

		// 完全同期(削除込みでの同期)の場合
		if ($ldap_config['ldapRunMode'] == 'sync') {
			$user_db = new passlogic_user(getPDO());

			// DBユーザ全件検索=>比較用uidリストに存在するか確認
			$conditions = array('domain' => $this->domain, 'tenant' => $this->tenant);
			$count_db_users = $user_db->count_user($conditions);
			$perPage = 1000;
			for ($i = 0; $i < $count_db_users / $perPage; $i++) {
				$db_users = $user_db->query_userlist($conditions, array('uid'), array('uid' => 'ASC'), $perPage, $i * $perPage);
				foreach ($db_users as $db_user) {
					if (!in_array($db_user['uid'], $ldap_users_uid)) {
						// 同期対象外ユーザを削除フラグを立ててcsvに追加(削除はcsvの行頭に書込み)
						file_put_contents($tmpfile_name, '"d","' . $db_user['uid'] . '","' . $this->domain . '"' . "\n", FILE_APPEND);
					}
				}
			}
		}

		// LDAPからの取り込み完了
		$this->plLog->log("31202", '', $this->domain, $this->tenant);

		// 最後にuserimportコマンドを実行して、作成したCSVファイルをもとにユーザデータ更新
		$cmd = '/usr/bin/php /opt/passlogic/apps/admin/cgi/userimport.php ';
		$cmd .= escapeshellarg($tmpfile_name) . ' ';
		$cmd .= escapeshellarg($config_file) . ' ';
		$cmd .= "'' "; // SERVER_ADDR
		$cmd .= "'' "; // REMOTE_ADDR
		$cmd .= "'' "; // HTTP_USER_AGENT
		$cmd .= escapeshellarg($this->domain) . ' ';
		$cmd .= escapeshellarg($targetTenant) . ' ';
		$cmd .= ' > /dev/null &';
		exec($cmd);
	}

}

$targetLdapDomain = $argv[1];
$targetTenant = $argv[2];
$ldapsync = new ldapsync();
$ldapsync->main($targetLdapDomain, $targetTenant);
?>
