<?php

// restore.sh が作ったlockfile がある場合は exit(1) する
require_once('/opt/passlogic/apps/lib/passlogic_lock.php');
if (PasslogicLock::cron_exists())
{
	exit(1);
}

//処理時間を無制限にする
set_time_limit(0);

/**
 * REQUEST_METHOD がある場合、ポリシー設定画面で、有効期限切れお知らせメールを
 * 「すぐにメール送信」ボタンが押下されたと判断
 *
 * restore.sh は userPassChgMail.php 実行中にリストアをしないようにプロセス存在確認を
 * する。 そのため httpd ではなく php プロセスとして見えるように自身を php コマンドで実行
 *
 * 「すぐにメール送信」はコールバック未指定の XMLHttpRequest。
 *   php コマンド終了後は何も返さず終了する(空文字列を返却)
 */
if (isset($_SERVER['REQUEST_METHOD'])) {

	// global_setting.php をrequire する代わりにここで定義
	define("DEFAULT_POLICY", 'Default Policy');
	
	// arg1
	$policy=DEFAULT_POLICY;
	if (isset($_GET['policy']) && $_GET['policy'] != DEFAULT_POLICY) {
		$policy = $_GET['policy'];
	}

	// arg2
	if (isset($_GET['target_tenant'])) {
		$target_tenant = $_GET['target_tenant'];
	} else {
		$target_tenant = '';
	}

	// arg3
	if (isset($_GET['admin_flag'])) {
		$admin_flag = 'true';
	} else {
		$admin_flag = 'false';
	}

	// arg4
	if (isset($_GET['day1']) && isset($_GET['day2']) && isset($_GET['passchgdays'])) {
		$day1 = $_GET['day1'];
		$day2 = $_GET['day2'];
		$passchgdays = $_GET['passchgdays'];
		$arg4="$day1,$day2,$passchgdays";
	}

	exec("/usr/bin/php /opt/passlogic/apps/admin/userPasschgMail.php '$policy' '$target_tenant' '$admin_flag' '$arg4'> /dev/null 2>&1");
	exit;
}

/**
 * settings
 */
require_once('/opt/passlogic/apps/lib/settings/global_setting.php');
require_once(LIB_DIR . '/settings/server_setting.php');

$plLog = new PasslogicLog();
$plinc = new passlogicInc();
$passlogic_config = new passlogic_config(getPDO());

$policy = DEFAULT_POLICY;
if (isset($argv[1]) && $argv[1] != $policy) {
	$policy = $argv[1];
}

$target_tenant = '';
if (isset($argv[2])) {
	$target_tenant = $argv[2];
}

if (isset($argv[3]) && $argv[3] == 'true') {
	$admin_flag = true;
} else {
	$admin_flag = false;
}

$is_web_request = false;	
if (isset($argv[4])) {
	$ary = explode(',', $argv[4]);
	if (count($ary) == 3 ) {
		$is_web_request = true;
		list($day1, $day2, $passchgdays) = $ary;
	} else {
		$plLog->log("34005", '', '', $target_tenant);
		return false;
	}
}

if ($admin_flag) {
	$passlogic_policy_conf = $passlogic_config->get_config('admin_policy', $policy, $target_tenant);
} else {
	$passlogic_policy_conf = $passlogic_config->get_config('policy', $policy, $target_tenant);
}
if (!$is_web_request) {
	if ($passlogic_policy_conf['isPasschgTimeup'] == 'false') {
		// パスワード有効期限が無制限
		$plLog->log("34003", '', '', $target_tenant);
		return false;
	}
	if ($passlogic_policy_conf['notice_passchg_hour'] == '--') {
		// メール通知設定が無効
		$plLog->log("34004", '', '', $target_tenant);
		return false;
	}
	if (!is_numeric($passlogic_policy_conf['notice_passchg_days_1']) || !is_numeric($passlogic_policy_conf['notice_passchg_days_2'])) {
		// 設定値が正しくない
		$plLog->log("34005", '', '', $target_tenant);
		return false;
	}
	$day1 = intval($passlogic_policy_conf['notice_passchg_days_1']);
	$day2 = intval($passlogic_policy_conf['notice_passchg_days_2']);
	$passchgdays = intval($passlogic_policy_conf['passchgtimeupdays']);
}
$time1 = $day1 * 60 * 60 * 24;
$time2 = $day2 * 60 * 60 * 24;

// パラメータのチェック
if (!is_numeric($passchgdays) || $passchgdays < 0 ||
	!is_numeric($day1) || $day1 < 0 || $time1 > $passchgdays ||
	!is_numeric($day2) || $day2 < 0 || $time2 > $passchgdays) {
	// パラメータの設定が正しくない
	$plLog->log("34006", '', '', $target_tenant);
} else {
	$range_min = time() - $passchgdays + min($time1, $time2);
	$range_max = $range_min + abs($time1 - $time2);

	if ($admin_flag) {
		$user_db = new passlogic_admin(getPDO());
	} else {
		$user_db = new passlogic_user(getPDO());
	}
	$rs_list = $user_db->get_user_list_for_notice_expire($policy, $range_min, $range_max, $target_tenant);
	$mail_count = count($rs_list);

	// 対象ユーザにメールを送信
	// 処理の開始を記録
	if ($admin_flag) {
		$plLog->log("34001", '', '', $target_tenant, array('additional_message' => 'admin policy:' . $policy));
	} else {
		$plLog->log("34001", '', '', $target_tenant, array('additional_message' => 'policy:' . $policy));
	}
	sleep(1);
	if ($mail_count > 0) {
		foreach ($rs_list as $user) {
			if (strlen($user['uemail']) == 0) {
				// メールアドレスが登録されていない
				$plLog->log('34008', $user['uid'], $user['domain'], $user['tenant']);
				continue;
			}
			$params = array(
			    "uid" => $user['uid'],
			    "domain" => $user['domain'],
			    "tenant" => $user['tenant'],
			    "type" => $admin_flag ? "admin" : "user",
			    "num" => null,
			    "client_info" => null
			);

			$ret = $plinc->sendnotice("passchg", $params);
			if ($ret) {
				// メール送信に成功 (失敗時はsendnotice()内で31014でログに書かれる)
				$plLog->log('34007', $user['uid'], $user['domain'], $user['tenant']);
			}
		}
	}
	sleep(1);
	// 処理の終了を記録
	if ($admin_flag) {
		$plLog->log("34002", '', '', $target_tenant, array('additional_message' => 'admin policy:' . $policy));
	} else {
		$plLog->log("34002", '', '', $target_tenant, array('additional_message' => 'policy:' . $policy));
	}
}
?>

