#!/usr/bin/php
<?php
// restore.sh が作ったlockfile がある場合は exit(1) する
require_once('/opt/passlogic/apps/lib/passlogic_lock.php');
if (PasslogicLock::cron_exists())
{
	exit(1);
}
require_once('/opt/passlogic/apps/lib/settings/global_setting.php');
require_once(LIB_DIR . '/settings/server_setting.php');
require_once(APPS_DIR . '/admin/config.inc.php');
require_once(LIB_DIR . '/database/databaseManager.inc.php');

class userDatabaseManager extends databaseManager
{
    function __construct(PDO $pdo)
    {
        parent::__construct($pdo);
    }

    public function fetch($days, $tenant)
    {
        // 日数のオプション指定
        $deadline = time() - 24 * 3600 * $days;

        // 対象となるカラム数を取得
        $rs1 = $this->execute(
            'select count(*) as count from passlogic_user' .
            ' where tenant=:tenant and passsetdate<=:deadline and upasschged=0 and udisabled=0 and not passsetdate=0',
            array(
                'tenant'   => $tenant,
                'deadline' => $deadline,
            )
        );
        if (!is_array($rs1)) {
            print("異常が発生しました。\n");
            exit(1);
        }
        $count = $rs1[0] ? $rs1[0]['count'] : 0;

        // 一度の処理上限を1000ユーザとし、クエリを分割する
        $perPage = 1000;
        $cond = $count / $perPage;
        $userList = [];

        for ($i = 0; $i < $cond; $i++) {
            $rs2 = $this->execute(
                'select uid, domain, tenant, passsetdate, upasschged, udisabled from passlogic_user' .
                ' where tenant=:tenant and passsetdate<=:deadline and upasschged=0 and udisabled=0 and not passsetdate=0' .
                ' order by uid asc limit :perPage offset :offset',
                array(
                    'tenant'   => $tenant,
                    'deadline' => $deadline,
                    'perPage'  => $perPage,
                    'offset'   => $i * $perPage
                )
            );

            if (!is_array($rs2)) {
                print("異常が発生しました。\n");
                exit(1);
            }

            foreach ($rs2 as $user) {
                $userList[$user['uid']] = $user;
            }
        }

        // print_r($userList);

        return $userList;
    }

    public function disable($userList)
    {
        // 1ユーザごとにクエリを発行する
        foreach ($userList as $key => $user) {
            $rs = $this->execute(
                'update passlogic_user set udisabled=1 where uid=:uid and domain=:domain and tenant=:tenant',
                array(
                    'uid' => $user['uid'],
                    'domain' => $user['domain'],
                    'tenant' => $user['tenant']
                )
            );
            if (!is_array($rs)) {
                print("異常が発生しました。\n");
                exit(1);
            }
        }
    }
}

/**
 * フラグファイルチェック
 */

$filename = DATA_DIR . '/conf/flag/main_server';

if (!file_exists($filename)) {
    print("{$filename}が存在しません。\n");
    exit(1);
}

/**
 * 引数チェック
 */

function usage()
{
    print("Usage: /opt/passlogic/apps/tools/disableUsersByPasschged.php <days> [<method>] [<tenant>]\n");
    print("  <days>    Natural number\n");
    print("  <method>  disabled/list\n");
    print("  <tenant>  tenant name (0 - 255)\n");
}

// 引数は 1 - 3
if (!(in_array($argc, range(1 +(1), 3 +(1), 1), true))) {
    usage();
    exit(1);
}

// 日数
$days = $argv[1];
if (!preg_match('/\A[1-9][0-9]*+\z/', $days)) {
    print("日数は自然数で指定してください。\n\n");
    usage();
    exit(1);
}

// 操作(disabled/list)
$method = ($argv[2]) ?: 'list';
if (!('list' === $method || 'disabled' === $method)) {
    print("未定義の操作が指定されました。\n\n");
    usage();
    exit(1);
}

$tenant = ($argv[3]) ?: '';
if(strlen($tenant) > 255){
    print("テナント名は255文字以下で指定してください\n\n");
    usage();
    exit(1);
}
/**
 * DB処理
 */
$mgr = new userDatabaseManager(getPDO());

$userList = $mgr->fetch($days, $tenant);

$hits = count($userList);

if ($method === 'list') {
    print("{$hits}件の対象ユーザが見つかりました。\n\n");
} else if ($method === 'disabled') {
    $mgr->disable($userList);
    print("{$hits}件の対象ユーザを無効化しました。\n\n");
} else {
    // 発生しない
}

print("[uid], [domain], [tenant], [最終パスワード変更日時]\n");
foreach ($userList as $user) {
    $date = date("Y/m/d H:i:s", $user['passsetdate']);
    print("{$user['uid']}, {$user['domain']}, {$user['tenant']}, {$date}\n");
}

exit(0);
?>
