#!/usr/bin/php
<?php
// restore.sh が作ったlockfile がある場合は exit(1) する
require_once('/opt/passlogic/apps/lib/passlogic_lock.php');
if (PasslogicLock::cron_exists())
{
	exit(1);
}

require_once('/opt/passlogic/apps/lib/settings/global_setting.php');
require_once(LIB_DIR . '/settings/server_setting.php');
require_once(APPS_DIR . '/admin/config.inc.php');

define('JSON_FILE', APPS_DIR . '/admin/tmpl/json/userimport.json');
define('XML_FILE', APPS_DIR.'/passlogic-lang.xml');

class getUserDataCSV
{
    public function __construct()
    {
        global $Language;
        $this->lang_arr = $Language->loadData(XML_FILE);

        $short = "p:l:d:t:n:";
        $long = array(
        "testuser",
        );
        $options = getopt($short, $long);

        $this->fpath = $options["p"] ? $options["p"] : TMP_DIR . "/passlogic.csv";
        $this->langType = $options["l"] ? $options["l"] : "ja";
        $this->fileType = $options["d"] == "up" ? "up" : "dl";
        $this->tenantType = $options["t"] ? $options["t"] : "sg";
        $this->tenantName = "";
        $this->testUser = false;
        if ($this->tenantType != "sg") {
            if ($options["n"] != "") {
                $this->tenantName = $options["n"];
            }
            if (isset($options["testuser"])) {
                $this->testUser = true;
            }
        }
    }

    public function main()
    {
        // パラメータ名称取得
        $this->getParamName();

        // csvファイルヘッダ設定
        $this->setCSVHeader();

        // 条件設定
        $this->setConditions();

        // データ取得
        $this->getData();
    }

    // パラメータ名称取得
    private function getParamName()
    {
        // ヘッダファイルセットを取得
        $header_json = file_get_contents(JSON_FILE);
        $header_arr = json_decode($header_json, true);

        $additional = array(
            "up" => array(                                          // アップロード
                "sg" => array('core', 'sg_core', 'sg_upload'),	    // シングルテナントの場合
                "ma" => array('core', 'ma_core', 'ma_upload'),	    // マルチテナント（マスターテナント）の場合
                "mt" => array('core', 'mt_core', 'mt_upload'),	    // マルチテナント（子テナント）の場合
            ),
            "dl" => array(                                          // ダウンロード
                "sg" => array('core', 'sg_core', 'sg_download'),	// シングルテナントの場合
                "ma" => array('core', 'ma_core', 'ma_download'),	// マルチテナント（マスターテナント）の場合
                "mt" => array('core', 'mt_core', 'mt_download'),	// マルチテナント（子テナント）の場合
            ),
        );

        // 予期せぬタイプの場合
        if (!array_key_exists($this->fileType, $additional)) {
            print("異常が発生しました。1\n");
            exit(1);
        }
        if (!array_key_exists($this->tenantType, $additional[$this->fileType])) {
            print("異常が発生しました。2\n");
            exit(1);
        }

        // タイプ毎の追加パラメータ設定
        $this->download_arr = array();
        $this->only_arr = array();
        foreach ($additional[$this->fileType][$this->tenantType] as $v) {
            if (strpos($v, "up")) {
                $this->only_arr = isset($header_arr[$v]) ? array_merge($this->only_arr, $header_arr[$v]) : $this->only_arr;
            } else {
                $this->download_arr = isset($header_arr[$v]) ? array_merge($this->download_arr, $header_arr[$v]) : $this->download_arr;
            }
        }
    }

    // csvファイルヘッダ設定
    private function setCSVHeader()
    {
        // 言語指定チェック
        if (!array_key_exists($this->langType, $this->lang_arr["LANG_MANUAL"])) {
            print("異常が発生しました。3\n");
            exit(1);
        }

        $csv_header_arr = array();
        foreach ($this->download_arr as $lang) {
            $csv_header_arr[] = $this->lang_arr[$lang][$this->langType];
        }
        foreach ($this->only_arr as $lang) {
            $csv_header_arr[] = $this->lang_arr[$lang][$this->langType] . "*";
        }
        $csv_header = '[';
        $csv_header .= implode('],[', $csv_header_arr);
        $csv_header .= "]\n";
        $csv_header = mb_convert_encoding($csv_header, "sjis-win", "UTF-8");

        file_put_contents($this->fpath, $csv_header);
    }

    // 条件設定
    private function setConditions()
    {
        // データダウンロードの場合、行わない
        if ($this->fileType != "dl") {
            return;
        }

        // DBから取得するカラム名(記述された順で出力される)
        unset($this->download_arr['delflag']);
        $this->getColumnOrder = array_keys($this->download_arr);

        $this->conditions = array('tenant' => $this->tenantName);

        // 通常テナント管理者は通常ユーザのみ取得できる(テストユーザを取得できないようにする)
        // （シングルモードの場合はテストユーザ取得できない）
        if (!$this->testUser) {
            $this->conditions['test_user'] = 0;
        }
    }

    // データ取得
    private function getData()
    {
        // データダウンロードの場合、行わない
        if ($this->fileType != "dl") {
            return;
        }

        // 時間制限解除
        if (!ini_get('safe_mode')) {
            set_time_limit(0);
        }

        // 一度の取得上限1000ユーザ
        $perPage = 1000;
        $user_db = new passlogic_user(getPDO());
        $count = $user_db->count_user($this->conditions);
        $cond = $count / $perPage;
        for ($i = 0; $i < $cond; $i++) {
            $rs = $user_db->query_userlist($this->conditions, $this->getColumnOrder, array('uid' => 'ASC'), $perPage, $i * $perPage);

            if (is_array($rs)) {
                foreach ($rs as $val) {

                    // DelFlag
                    $csv = ",";

                    foreach ($this->getColumnOrder as $columnName) {
                        switch ($columnName) {
                            case 'uexpiry':
                                if ($val[$columnName]) {
                                    $csv .= "\"" . str_replace("\"", "\"\"", date("Ymd", $val[$columnName])) . "\",";
                                } else {
                                    $csv .= "\"\",";
                                }
                                break;

                            case 'lastauthdate':
                                if ($val[$columnName]) {
                                    $csv .= "\"" . str_replace("\"", "\"\"", date("Y/m/d H:i:s", $val[$columnName])) . "\",";
                                } else {
                                    $csv .= "\"\",";
                                }
                                break;

                            case 'passsetdate':
                                if ($val[$columnName]) {
                                    $csv .= "\"" . str_replace("\"", "\"\"", date("Y/m/d H:i:s", $val[$columnName])) . "\",";
                                } else {
                                    $csv .= "\"\",";
                                }
                                break;

                            default:
                                $csv .= "\"" . str_replace("\"", "\"\"", $val[$columnName]) . "\",";
                                break;
                        }
                    }
                    // 最終行の改行追加
                    $csv .= "\n";

                    $csv = mb_convert_encoding($csv, "sjis-win", "UTF-8");
                    file_put_contents($this->fpath, $csv, FILE_APPEND);
                }
            }
        }
    }
}

$g = new getUserDataCSV();
$g->main();

?>
