#!/usr/bin/php
<?php
// restore.sh が作ったlockfile がある場合は exit(1) する
require_once('/opt/passlogic/apps/lib/passlogic_lock.php');
if (PasslogicLock::cron_exists())
{
	exit(1);
}

@date_default_timezone_set("Asia/Tokyo");
define("APPS_DIR", "/opt/passlogic/apps");
define("LIB_DIR", APPS_DIR . "/lib");
define("PEAR_DIR", LIB_DIR . "/pear");

require_once(LIB_DIR . '/database/passlogic_user.php');
require_once(LIB_DIR . '/database/passlogic_config.php');
require_once(LIB_DIR . "/ldap.inc.php");
require_once(LIB_DIR . "/crypt.inc.php");
require_once(LIB_DIR . "/log.inc.php");
require_once(LIB_DIR . "/passlogic_api_util.inc.php");
require_once(LIB_DIR . "/model/user_model.php");

class passlogic_adsync {

	var $progname = "passlogic_adsync";

	/**
	 * コンストラクタ
	 */
	function __construct() {
		// 各種インスタンスの準備
		$this->plau = new passlogic_api_util();
		$this->crypt = new Crypt;
		$this->plLog = new PasslogicLog();
		$this->log_code = $this->plLog->_get_log_xml();
		$this->users_db = new passlogic_user(getPDO());
		$this->passlogic_config = new passlogic_config(getPDO());
	}

	// プログラム本体
	function main() {
		// 開始ログ
		$this->plLog->log('31101', $this->progname, 'ALL', '');
		$this->to_logfile($this->log_code['31101']['message'] . "(31101)");

		// 全domain設定読み込み
		$domain_list = $this->passlogic_config->get_config_list('domain', NULL, true);

		// LDAP認証連携ドメインを抽出し、ユーザを削除
		foreach ($domain_list as $domain_conf) {
			/**
			 * Phase 1: ldapと接続
			 */
			// ドメイン設定にLDAP認証連携があるかチェック
			if ($domain_conf['config']['domaintype'] != 'auth') {
				// LDAP 認証連携ではないのでSKIP
				continue;
			}

			// LDAPと接続
			$ldap = $this->bind_ldap($domain_conf);
			if (!$ldap) {
				// 接続失敗
				$this->to_logfile("ldap bind error. domain=" . $domain . " ALL SERVERS");
				continue;
			}

			// 頻出変数のエイリアスを作成
			$domain = $domain_conf['config']['domainname'];
			$tenant = $domain_conf['tenant'];

			/**
			 * Phase 2: PassLogicのユーザを取得し、LDAPに存在の有無を問い合わせる
			 */
			$conditions = array('domain' => $domain, 'tenant' => $tenant);
			$params_column = array('uid');
			$domain_total = $this->users_db->count_user($conditions);

			// ドメインにユーザなし
			if ($domain_total == 0 || $domain_total == -1) {
				continue;
			}

			// $MasPageSize分ずつ、DBより取得
			$MaxPageSize = 100;
			for ($i = 0; $i < $domain_total / $MaxPageSize; $i++) {
				$userList = $this->users_db->query_userlist($conditions, $params_column, array('uid' => 'ASC'), $MaxPageSize, $i * $MaxPageSize);
				if (!$userList) {
					// ユーザ取得失敗（DBエラー）
					$this->to_logfile("Failed to read passlogic users. domain=" . $domain . ", offset=" . $offset);
					break;
				} elseif (count($userList) <= 0) {
					break;
				}

				// 取得したユーザをLDAPでチェック
				foreach ($userList as $passlogicuser) {
					if ($this->is_exist_LDAP($passlogicuser['uid'], $ldap, $domain_conf['config']) === FALSE) {
						/**
						 * Phase 4: LDAPに存在しないので削除
						 */
						$user = new user_model($passlogicuser['uid'], $domain, $tenant);
						$ret = $this->plau->userdelete($user);
						$code = $ret['code'];
						$message = $this->log_code[$code]['message'] . "({$code})";
						if ($code == '30003') {
							$this->to_logfile("user deleted., " . $passlogicuser['uid'] . "@" . $domain . ", " . $message);
						} else {
							$this->to_logfile("userdelete error., " . $passlogicuser['uid'] . "@" . $domain . ", " . $message);
						}
					}
				}
			}
		}

		// 完了ログ
		$this->plLog->log('31102', $this->progname, 'ALL', '');
		$this->to_logfile($this->log_code['31102']['message'] . "(31102)");
	}

	/**
	 * LDAPと接続する
	 */
	private function bind_ldap($domain_conf) {
		// LDAPのパスワードを読解
		$ldapbindpasswd = $this->crypt->decrypt_string($domain_conf['config']['ldapBindpasswd'], $domain_conf['config']['domainname']);

		// ldap操作クラス
		$host = array($domain_conf['config']['ldapHostname'], $domain_conf['config']['ldapHostname2']);
		$hostArr = array_filter($host, "strlen");
		$bind_done = false;
		foreach ($hostArr as $key => $val) {
			$ldap = new ldap($domain_conf['tenant']);
			if (!$ldap->_bind($val, $domain_conf['config']['ldapPort'], $domain_conf['config']['ldapBinddn'], $ldapbindpasswd, $domain_conf['config']['ldapTopObject'], $domain_conf['config']['ldapSecure'])) {
				$this->to_logfile("ldap bind error. domain=" . $domain_conf['config']['domainname'] . " server=" . $val);
				continue;
			} else {
				$ldapHost = $val;
				$bind_done = true;
				break;
			}
		}

		// レスポンス
		if ($bind_done) {
			// 接続成功
			return $ldap;
		} else {
			// 接続失敗
			return NULL;
		}
	}

	/**
	 * 専用ログファイルへ出力する
	 */
	private function to_logfile($string) {
		file_put_contents("/var/log/passlogic/passlogic_adsync.log", date("Y/m/d H:i:s") . " " . $string . "\n", FILE_APPEND | LOCK_EX);
	}

	/**
	 * $uidのユーザが$ldapに存在するかチェック
	 * 存在するときはTRUE、存在しないときはFALSE、
	 * エラー時はNULLをリターン
	 */
	private function is_exist_LDAP($uid, $ldap, $ldapconfig) {
		// ユーザ検索用のフィルタを設定
		if ($ldapconfig['ldapType'] == 'ActiveDirectory') {
			$filter = "(objectclass=user)";
		} else if ($ldapconfig['ldapType'] == 'OpenLDAP') {
			$filter = "";
		} else {
			return NULL;
		}

		// 
		$tmpfilter = "(" . $ldapconfig['ldapAttrUid'] . "=" . $uid . ")";
		if (strlen($tmpfilter) > 0) {
			$filter .= "(|" . $tmpfilter . ")";
		}
		if ($ldapconfig['ldapFilter']) {
			$filter = "(&{$ldapconfig['ldapFilter']}{$filter})";
		}
		$ldapusers = $ldap->_search($ldapconfig['ldapTopObject'], $filter, $ldapconfig['ldapAttrUid'], $ldapconfig['ldapAttrUemail']);

		// LDAPエラーのチェック
		if ($ldapusers === FALSE) {
			if ($ldap->err) {
				$this->to_logfile($ldap->err . " domain=" . $ldapconfig['domainname']);
			} else {
				$this->to_logfile("ldap search error. domain=" . $ldapconfig['domainname']);
				$this->to_logfile($uid);
			}
			return NULL;
		}

		// フィルターに合致するユーザの数をチェック
		if (empty($ldapusers)) {
			// 合致するユーザはいなかった
			return FALSE;
		}

		// 合致するユーザがいた
		return TRUE;
	}

}

$adsync = new passlogic_adsync();
$adsync->main();
?>
