#!/bin/bash

PASSLOGIC_CRON_LOCK=passlogic_cron.lock
SESSION_TRUNCATION_ON_ERROR=1
SESSION_GC_DAYS=3
LOG=/var/log/passlogic/passlogic_session_gc.log

log(){
    echo `date "+%Y-%m-%d %H:%M:%S"` $* >> $LOG
}

check_lock_file() {
	if [ ! -e /opt/passlogic/data/lock/$PASSLOGIC_CRON_LOCK ]
	then
		return 0
	else
		return 1
	fi
}

check_main_server_flag() {
	if [ -e /opt/passlogic/data/conf/flag/main_server ]
	then
		return 1
	else
		return 0
	fi
}

border() {
    local border
    border=`date "+%s" -d "-${SESSION_GC_DAYS} days"`
    echo $border
}

delete_session_records() {
	query="delete from passlogic_session where updated_at < `border`"
	result=`su - passlogic -c "/opt/passlogic/pgsql/bin/psql -h localhost -p 9915 -U passlogic -c '$query' 2>&1"`
    return $?
}

truncate_passlogic_session_table() {
	query="truncate passlogic_session"
	result=`su - passlogic -c "/opt/passlogic/pgsql/bin/psql -h localhost -p 9915 -U passlogic -c '$query' 2>&1"`
	return $?
}

check_lock_file
if [ $? -eq 1 ]
then
	log LOG: $PASSLOGIC_CRON_LOCK exists.
	exit 1
fi

check_main_server_flag
if [ $? -eq 1 ]
then
	log LOG: this authentication server is not the main server.
	exit 0
fi

log LOG: Started deleting old session records in passlogic_session table...

delete_session_records
if [ $? -eq 0 ]
then
	log LOG: Finished deleting records updated_at older than ${SESSION_GC_DAYS} days: $result
else
	log ERROR: Failed deleting records updated_at older than ${SESSION_GC_DAYS} days: $result -- query was \'$query\'.

	if [ ${SESSION_TRUNCATION_ON_ERROR} -eq 1 ]
	then
		truncate_passlogic_session_table
		if [ $? -eq 0 ]
		then
			log LOG: Truncated passlogic_session table.
		else
			log ERROR: Failed truncating passlogic_session table: $result
		fi
	fi
fi
