#!/bin/bash

PASSLOGIC_CRON_LOCK=passlogic_cron.lock
PASSLOGIC_PDO_LOCK=pdo.lock
PATH=/opt/passlogic/apps/tools/:$PATH
pid=$$

trap "make_lock.sh $pid ${PASSLOGIC_CRON_LOCK} restore.sh -unlock" 0
make_lock.sh $pid ${PASSLOGIC_CRON_LOCK} restore.sh
[ $? -ne 0 ] && exit 4

timeout 300 bash -c 'while passlogic_cron_mon.sh ; do sleep 5; done'
[ $? -ne 0 ] && exit 4

trap "make_lock.sh $pid ${PASSLOGIC_PDO_LOCK} restore.sh -unlock; make_lock.sh $pid ${PASSLOGIC_CRON_LOCK} restore.sh -unlock" 0
make_lock.sh $pid ${PASSLOGIC_PDO_LOCK} restore.sh
[ $? -ne 0 ] && exit 4

FILENAME="passlogicbackup"

pm=/opt/passlogic/apps/tools/pm.sh

call_pm() { 
	echo $*
	echo $* | $pm | tail -1 | grep -q -E '^:ok'
	[ ! $? -eq 0 ] && echo :ng && exit 1 
	echo :ok
}

remote_node_alive() {
	echo remote_ping | $pm | tail -1 | grep -q -E '^:ok'
}

# o sync a file in /opt/passlogic/tmp/$FILENAME/opt/passlogic/data/ with /opt/passlogic/data/
# o change a file owner/group to a specified owner/group
fsync() {
	local file=$1
	local owner_group=$2
	( ( [ ! -e /opt/passlogic/tmp/$FILENAME/opt/passlogic/data/${file} ] && rm -f /opt/passlogic/data/${file} ) || \
	  ( cp -f --preserve=timestamps /opt/passlogic/tmp/$FILENAME/opt/passlogic/data/${file} /opt/passlogic/data/${file} && \
	chown ${owner_group} /opt/passlogic/data/${file}) )
}

# パスワード入力チェック
if [ -z $1 ]; then
	echo "please enter password"
	exit 1
fi

# DBリカバリ・confリカバリのフラグ
case "$2" in
	-d)
		db=1
		conf=0
		;;
	-c)
		db=0
		conf=1
		;;
	*)
		db=1
		conf=1
		;;
esac

# tmpに作業フォルダ作成
rm -Rf /opt/passlogic/tmp/$FILENAME
rm -f /opt/passlogic/tmp/$FILENAME.tar.gz
mkdir /opt/passlogic/tmp/$FILENAME

# 作業フォルダ内にバックアップファイルの中身を展開1
cd /opt/passlogic/tmp
unzip -o -P $1 /opt/passlogic/tmp/$FILENAME.zip
if [ $? -ne 0 ]; then
	exit 1
fi

# 作業フォルダ内にバックアップファイルの中身を展開2
if [ -e /opt/passlogic/tmp/$FILENAME.tar.gz ]; then
	tar zxvf /opt/passlogic/tmp/$FILENAME.tar.gz
else
	exit 2
fi

# confリカバリフラグが立っている時は，以下を配置
#
#   1) conf/flag/passlogic_config
#   2) conf/redundant.conf
#   3) license-ent.asc
#   4) logo_images.png
#
#   1), 4) は data.tar.gz に含まれない場合がある。
#   その場合で、もしリストア先に該当ファイルが存在したら削除する。

if [ $conf -eq 1 ]; then
	# data.tar.gzを展開
	if [ -e /opt/passlogic/tmp/$FILENAME/data.tar.gz ]; then
		tar zxvf /opt/passlogic/tmp/$FILENAME/data.tar.gz -C /opt/passlogic/tmp/$FILENAME/
	else
		exit 2
	fi

	fsync conf/flag/passlogic_config apache:apache
	fsync conf/redundant.conf passlogic:passlogic
	fsync license-ent.asc apache:apache
	fsync logo_images.png apache:apache
fi

# dbリカバリフラグが立っている時はdbbackup.sqlを流し込む
if [ $db -eq 1 ]; then
    export PGUSER="passlogic"
    export PGDATABASE="passlogic"

    /opt/passlogic/pgsql/bin/psql -h localhost -p 9915 passlogic  < /opt/passlogic/tmp/$FILENAME/dbbackup.sql
    ret_psql=$?
    unset PGUSER
    unset PGDATABASE
fi

make_lock.sh $pid ${PASSLOGIC_PDO_LOCK} restore.sh -unlock
trap "make_lock.sh $pid ${PASSLOGIC_CRON_LOCK} restore.sh -unlock" 0

if [ $ret_psql -ne 0 ]
then
    exit 3
fi

call_pm recreate_setting
call_pm radiusd_restart
call_pm httpd_graceful

other_server=/opt/passlogic/data/conf/flag/other_server

if [ -e $other_server ];then
	if remote_node_alive
	then
		call_pm remote_recreate_setting
		call_pm remote_radiusd_restart
		call_pm remote_httpd_graceful
	fi
fi

call_pm reloadTenant

# tmp内の作業フォルダを削除
rm -Rf /opt/passlogic/tmp/$FILENAME.tar.gz
rm -Rf /opt/passlogic/tmp/$FILENAME.zip
rm -Rf /opt/passlogic/tmp/$FILENAME

# キャッシュを削除
rm -Rf /opt/passlogic/tmp/*.cache
